@@ -27,7 +27,7 @@ __copyright__ = "Copyright 2020, Inria"
 __credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
                "Alexis Thibault", "Tobias Van Baarsel"]
 __license__ = "GPL 3.0"
-__version__ = "0.7.2"
+__version__ = "0.8.0"
 __email__ = "openwind-contact@inria.fr"
 __status__ = "Dev"
 
@@ -36,3 +36,5 @@ from .technical import InstrumentGeometry, Player
 from .continuous import InstrumentPhysics, Physics
 from .impedance_computation import ImpedanceComputation
 from .temporal_simulation import simulate
+from .frequential import FrequentialSolver
+from .temporal import TemporalSolver
