@@ -67,7 +67,8 @@ from .radiation_model import radiation_model
 from .junction import (PhysicalJunction,
                        JunctionTjoint,
                        JunctionSimple,
-                       JunctionDiscontinuity)
+                       JunctionDiscontinuity,
+                       JunctionSwitch)
 
 # - Excitators -
 from .excitator import (create_excitator, Excitator, Flow, Flute, Reed1dof)
