@@ -164,13 +164,37 @@ class InstrumentPhysics:
         self.__build_netlist()
         self.netlist.set_fingering_chart(instrument_geometry.fingering_chart)
 
+
+        _, (input_end,) = self.netlist.get_connector_and_ends(self.source_label)
+        self._first_pipe = input_end.get_pipe()
         if nondim:
-            input_end = self.netlist.get_connector_and_ends(self.source_label)[1]
-            pipe_ref = input_end[0].get_pipe()
-            self.scaling.set_nondimensionalization(pipe_ref)
+            self.scaling.set_nondimensionalization(self._first_pipe)
 
         self.__check()
 
+    def get_entry_coefs(self, *labels):
+        """Obtain values of physical properties of air at the entry point.
+
+        Parameters
+        ----------
+        *labels : string...
+            the names of the coefficients to take.
+            See :py:class:`Physics <openwind.continuous.physics.Physics>`
+
+        Returns
+        -------
+        values : tuple of (float or array-like)
+
+        Example
+        -------
+
+        .. code-block:: python
+
+           instru_phy.get_entry_coefs('Cp', 'c', 'gamma', 'kappa', 'mu', 'rho', 'temp', 'T')
+
+        """
+        return self._first_pipe.get_physics().get_coefs(0, *labels)
+
     def rad_model(self, label, pipe_end):
         """
         Get the radiation model corresponding to a given opening
