@@ -76,7 +76,7 @@ class JunctionDiscontinuity(PhysicalJunction):
         \\end{align}
 
     The equations at the junction are the flow conservation :math:`u_1 + u_2=0`
-    and :math:`p_1 = p_1 + m_{disc} \\partial_t u_1`. In openwind, it is
+    and :math:`p_2 = p_1 + m_{disc} \\partial_t u_1`. In openwind, it is
     rewritten as a linear system by introducing an additional variable
     :math:`\\gamma`:
 
@@ -124,7 +124,7 @@ class JunctionDiscontinuity(PhysicalJunction):
         """
         return self.scaling.get_impedance() * self.scaling.get_time()
 
-    def compute_masse(self, r1, r2, rho):
+    def compute_mass(self, r1, r2, rho):
         """
         Compute mass of a Junction from physical parameters.
 
@@ -188,7 +188,7 @@ class JunctionDiscontinuity(PhysicalJunction):
         dmass = (6*.09616*alpha**5 - 5*.12386*alpha**4 + 4*0.03816*alpha**3
                 + 3*0.0809*alpha**2 - .353)*dalpha
 
-        mass = self.compute_masse(r1, r2, rho)
+        mass = self.compute_mass(r1, r2, rho)
         return rho/rmin*dmass/coef_scaling_masses - drmin/rmin*mass
 
 class JunctionTjoint(PhysicalJunction):
@@ -514,3 +514,129 @@ class JunctionTjoint(PhysicalJunction):
         dM = np.array([dm_minus, dm_plus])
         dT = np.array([[0, 0, 0], [0, 0, 0]])
         return dM, dT
+
+class JunctionSwitch(PhysicalJunction):
+    r"""
+    Models the switch junction between 3 pipes with discontinuity of sections.
+
+    Following the opening state of the switch the acoustic waves are oriented
+    from pipe1 to pipe2 (open switch) or from pipe 1 to pipe 3 (closed switch).
+    Intermediate state are accepted.
+    An acoustic mass can be added in case of discontinuity of sections
+    (cf. :py:class:`JunctionDiscontinuity<JunctionDiscontinuity>`).
+
+    The equations at the junction are the flow conservation
+
+    .. math::
+        \begin{cases}
+        u_1 - u_2 - u_3 =0 \\
+        p_1 = \alpha p_2 + (1 - \alpha) p_3 - \left(\alpha m_{12} + (1-\alpha) m_{13} \right) \partial_t u_1
+        \end{cases}
+
+    And :math:`u_2=\alpha u_1`; :math:`u_3=(1 - \alpha) u_1`.
+    In openwind, it is rewritten as a linear system by introducing an additional
+    variable :math:`\gamma`:
+
+    .. math::
+        \begin{cases}
+        \left( \begin{smallmatrix} u_1 \\ - u_2 \\ -u_3 \end{smallmatrix} \right) = T^{\star} \gamma \\
+        M \partial_t \gamma + T \left( \begin{smallmatrix} p_1 \\ p_2 \\ p_3 \end{smallmatrix} \right) = 0
+        \end{cases}
+
+    where :math:`T=(1, -\alpha, -(1 - \alpha))` and
+    :math:`M=\left(\alpha m_{12} + (1-\alpha)m_{13} \right)` with
+    :math:`m_{12}, m_{13}` the masses due to cross-section discontinuity between
+    pipes 1 and 2 and pipes 1 and 3 respectively
+
+
+    Parameters
+    ----------
+    label : string
+        The label of the junction
+    scaling : :py:class:`Scaling <openwind.continuous.scaling.Scaling>`
+        Nondimensionalization coefficients.
+    convention : {'PH1', 'VH1'}, optional
+        The convention used in this junction. The default is 'PH1'.
+    discontinuity_mass : boolean, optional.
+        If False, the masses are set to 0. Default is False.
+
+    """
+
+    def __init__(self, label, scaling, convention='PH1', discontinuity_mass=False):
+        super().__init__(label, scaling, convention)
+        self.discontinuity_mass = discontinuity_mass
+        self.disc_junc = JunctionDiscontinuity(label, scaling, convention)
+
+    def compute_masses(self, r1, r2, r3, rho, opening_factor):
+        """
+        Compute masses of the Junction from physical parameters.
+
+        Parameters
+        ----------
+        r1 : float
+            Radius of the 1st pipe, at the point of junction
+        r2 : float
+            Radius of the 2nd pipe, at the point of junction
+        r2 : float
+            Radius of the 3rd pipe, at the point of junction
+        rho : float
+            Air density, at the point of the junction
+        opening_factor : float within [0, 1]
+            The switch state.
+
+            - 1: the switch is "open". The pipe 1 is connected to pipe 2
+            - 2: the switch is "closed". The pipe 1 is connected to pipe 3
+
+        Returns
+        -------
+        float
+            The scaled acoustic mass associated to the junction.
+
+        """
+        T_J = np.array([[1, -opening_factor, opening_factor-1]])
+        if self.discontinuity_mass:
+            m12 = self.disc_junc.compute_mass(r1, r2, rho)
+            m13 = self.disc_junc.compute_mass(r1, r3, rho)
+            M_J = np.array([[m13 + opening_factor*(m12 - m13)]])
+        else:
+            M_J = np.array([[0.]])
+        return M_J, T_J
+
+    def get_diff_mass(self, r1, r2, r3, rho, opening_factor, dr1, dr2, dr3):
+        """
+        Derivate the mass w.r. to the radii.
+
+        Parameters
+        ----------
+        r1 : float
+            Radius of the 1st pipe, at the point of junction
+        r2 : float
+            Radius of the 2nd pipe, at the point of junction
+        r2 : float
+            Radius of the 3rd pipe, at the point of junction
+        rho : float
+            Air density, at the point of the junction
+        opening_factor : float within [0, 1]
+            The switch state.
+
+            - 1: the switch is "open". The pipe 1 is connected to pipe 2
+            - 2: the switch is "closed". The pipe 1 is connected to pipe 3
+        dr1 : float
+            The derivative of the first pipe radius.
+        dr2 : float
+            The derivative of the second pipe radius.
+        dr2 : float
+            The derivative of the third pipe radius.
+
+        Returns
+        -------
+        float
+            The derivative of the mass.
+
+        """
+        if self.discontinuity_mass:
+            dm12 = self.disc_junc.get_diff_mass(r1, r2, rho, dr1, dr2)
+            dm13 = self.disc_junc.get_diff_mass(r1, r3, rho, dr1, dr3)
+            return dm13 + opening_factor*(dm12 - dm13)
+        else:
+            return 0.
