@@ -20,26 +20,42 @@
 #
 # For more informations about authors, see the CONTRIBUTORS file
 
+from abc import ABC, abstractmethod
+import warnings
+
 import numpy as np
 import scipy.special as sp
-from abc import ABC, abstractmethod
 
 def losses_model(losses=True):
     """Create the right thermoviscous model.
 
     Parameters
     ----------
-    losses : {False, True, 'bessel', 'diffrepr', 'wl', 'keefe' ,'minikeefe'}
+    losses : {False, True, 'bessel', 'diffrepr', 'wl', 'keefe' ,'minikeefe',
+              'bessel_new', 'sh'}
         Which model of losses to use.
         False means lossless, True is the same as 'bessel'.
+        'bessel_new' : Zwikker-Koster model with modified loss coefficients to
+        account for conicity. See [Thibault1]_.
+        'sh' : loss coefficients in a cone expressed using the equation
+        of Spherical Harmonics. See [Thibault1]_.
         'diffrepr' stands for 'diffusive representation', which approximates
         the 'bessel' model but can be simulated in the time domain.
         'wl' stands for "Webster-Lokshin"
 
+
     Returns
     -------
     losses_model : :py:class:`ThermoviscousModel \
     <openwind.continuous.thermoviscous_models.ThermoviscousModel>`
+
+
+    References
+    ----------
+    .. [Thibault1] Thibault, Alexis and Chabassier, Juliette and Boutin, Henri
+        and Hélie, Thomas. A low reduced frequency model for viscothermal wave
+        propagation in conical tubes of arbitrary cross-section.
+        To be published.
     """
 
     if isinstance(losses, ThermoviscousModel):
@@ -48,7 +64,7 @@ def losses_model(losses=True):
     if losses in [False]:
         return ThermoviscousLossless()
     if losses in [True, 'bessel']:
-        return ThermoviscousBessel()
+        return ThermoviscousBessel(use_new_stokes_number=False)
     if losses.startswith('diffrepr'):
         return ThermoviscousDiffusiveRepresentation(losses)
     if losses == 'wl':
@@ -57,6 +73,10 @@ def losses_model(losses=True):
         return Keefe()
     if losses == 'minikeefe':
         return MiniKeefe()
+    if losses == 'sh':
+        return SphericalHarmonics()
+    if losses == 'bessel_new':
+        return ThermoviscousBessel(use_new_stokes_number=True)
 
     raise ValueError("`losses` argument should be one of: "
                      "{False, True (default), 'bessel' (same as True),"
@@ -87,6 +107,19 @@ def _sqrt_omegas(pipe, omegas_scaled):
     omegas = omegas_scaled / pipe.get_scaling().get_time()
     return np.sqrt(omegas)[:, np.newaxis]
 
+def calculate_hydraulic_radius(pipe, x):
+    """Calculate the hydraulic radius of the spherical cap at abscissa x."""
+    conicity = pipe.get_conicity_at(x)
+    Theta = np.arctan(conicity)
+    cosTheta = np.cos(Theta)
+    geomRadius = pipe.get_radius_at(x)
+    # Hydraulic radius
+    with warnings.catch_warnings():
+        sphereRadius = geomRadius/np.sin(Theta)
+        hydraulicRadius = sphereRadius * Theta**2*np.sin(Theta)/(2*(1-cosTheta))
+    # When Theta is too small, use the asymptotic value
+    return np.where(abs(Theta) < 1e-4, geomRadius, hydraulicRadius)
+
 
 class ThermoviscousModel(ABC): # Abstract Base Class
     """
@@ -297,21 +330,36 @@ class ThermoviscousBessel(ThermoviscousModel):
     Note that the lossless part is included in the cited equations,
     but excluded from this computation.
 
+    Option "new_stokes_number" allows the use of the hydraulic radius in the
+    formula for the Stokes number, as proposed in [Thibault]_,  instead of the
+    planar slice radius.
+
     References
     ----------
     .. [Chaigne_ZK] Chaigne, A., & Kergomard, J. (2016). "Acoustics of musical\
         instruments". Springer New York.
+    .. [Thibault] Thibault, Alexis and Chabassier, Juliette and Boutin, Henri
+        and Hélie, Thomas. A low reduced frequency model for viscothermal wave
+        propagation in conical tubes of arbitrary cross-section.
+        To be published.
     """
 
+    def __init__(self, use_new_stokes_number):
+        self.use_new_stokes_number = use_new_stokes_number
+
     def __str__(self):
         return "Zwikker and Kosten losses with Bessel function"
 
-    @staticmethod
-    def get_loss_flow_at(pipe, x, omegas_scaled):
+    def get_loss_flow_at(self, pipe, x, omegas_scaled):
         sqrt_omegas = _sqrt_omegas(pipe, omegas_scaled)
 
         rhos, mus = pipe.get_physics().get_coefs(x, 'rho', 'mu')
-        radius = pipe.get_radius_at(x)
+
+        # Choose geometric radius or hydraulic radius
+        if self.use_new_stokes_number:
+            radius = calculate_hydraulic_radius(pipe, x)
+        else:
+            radius = pipe.get_radius_at(x)
 
         kvr = (radius * np.sqrt(-1j*rhos/mus)) * sqrt_omegas
         coefZ = -2 * sp.jve(1, kvr) / (kvr * sp.jve(2, kvr))  # = Jv / (1 - Jv)
@@ -320,12 +368,16 @@ class ThermoviscousBessel(ThermoviscousModel):
         coef_loss_flow = coef_flow * coefZ
         return coef_loss_flow
 
-    @staticmethod
-    def get_loss_pressure_at(pipe, x, omegas_scaled):
+    def get_loss_pressure_at(self, pipe, x, omegas_scaled):
         sqrt_omegas = _sqrt_omegas(pipe, omegas_scaled)
         rhos, kappas, Cp, gamma = \
             pipe.get_physics().get_coefs(x, 'rho', 'kappa', 'Cp', 'gamma')
-        radius = pipe.get_radius_at(x)
+
+        # Choose geometric radius or hydraulic radius
+        if self.use_new_stokes_number:
+            radius = calculate_hydraulic_radius(pipe, x)
+        else:
+            radius = pipe.get_radius_at(x)
 
         ktr = (radius * np.sqrt(-1j*rhos*Cp/kappas)) * sqrt_omegas
         coefY = 2 * sp.jve(1, ktr) / (ktr * sp.jve(0, ktr))
@@ -334,8 +386,10 @@ class ThermoviscousBessel(ThermoviscousModel):
         coef_loss_pressure = coef_pressure * (gamma - 1) * coefY
         return coef_loss_pressure
 
-    @staticmethod
-    def get_diff_loss_flow(pipe, x, omegas_scaled, diff_index):
+    def get_diff_loss_flow(self, pipe, x, omegas_scaled, diff_index):
+        if self.use_new_stokes_number:
+            raise NotImplementedError()
+
         sqrt_omegas = _sqrt_omegas(pipe, omegas_scaled)
         rhos, mus = pipe.get_physics().get_coefs(x, 'rho', 'mu')
         radius = pipe.get_radius_at(x)
@@ -351,8 +405,10 @@ class ThermoviscousBessel(ThermoviscousModel):
         diff_loss_flow = (diff_coef_flow*jv + diff_radius*coef_flow*kv*jvprime)
         return diff_loss_flow
 
-    @staticmethod
-    def get_diff_loss_pressure(pipe, x, omegas_scaled, diff_index):
+    def get_diff_loss_pressure(self, pipe, x, omegas_scaled, diff_index):
+        if self.use_new_stokes_number:
+            raise NotImplementedError()
+
         sqrt_omegas = _sqrt_omegas(pipe, omegas_scaled)
         rhos, kappas, Cp, gamma = \
             pipe.get_physics().get_coefs(x, 'rho', 'kappa', 'Cp', 'gamma')
@@ -609,7 +665,7 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
     first-order rational fractions.
     This model is suited to time-domain simulation.
 
-    More detail in [Thibault]_.
+    More detail in [Thibault2]_.
 
     See Also
     --------
@@ -627,7 +683,7 @@ class ThermoviscousDiffusiveRepresentation(ThermoviscousModel):
 
     References
     ----------
-    .. [Thibault] Thibault, A. and Chabassier, J. "Dissipative time-domain 1D
+    .. [Thibault2] Thibault, A. and Chabassier, J. "Dissipative time-domain 1D
         model for viscothermal acoustic propagation in wind instruments".
         To be published.
 
@@ -785,7 +841,9 @@ class WebsterLokshin(ThermoviscousModel):
         Matignon,  Denis. "One-dimensional acoustic models of horns and
         comparison with measurements". (2013) Acta Acustica united with
         Acustica, vol. 99 (n° 6). pp. 960-974. ISSN 1610-1928
-    .. [RR] Alexis Thibault, Juliette Chabassier. Viscothermal models for wind musical instruments. [Research Report] RR-9356, Inria Bordeaux Sud-Ouest. 2020. ⟨hal-02917351⟩
+    .. [RR] Alexis Thibault, Juliette Chabassier. Viscothermal models for wind
+        musical instruments. [Research Report] RR-9356, Inria Bordeaux
+        Sud-Ouest. 2020. ⟨hal-02917351⟩
     """
 
     def __repr__(self):
@@ -817,3 +875,101 @@ class WebsterLokshin(ThermoviscousModel):
 
     def get_diff_loss_pressure(self, pipe, x, omegas_scaled, diff_index):
         raise NotImplementedError()  # TODO ?
+
+
+class SphericalHarmonics(ThermoviscousModel):
+    """Boundary layer losses in a cone.
+
+    Generalization of Zwikker--Kosten coefficients for
+    a cylinder, as in [Thibault3]_.
+
+    References
+    ----------
+    .. [Thibault3] Thibault, Alexis and Chabassier, Juliette and Boutin, Henri
+        and Hélie, Thomas. A low reduced frequency model for viscothermal wave
+        propagation in conical tubes of arbitrary cross-section.
+        To be published.
+    """
+
+    def __init__(self):
+        """Use library mpmath to compute hypergeometric function."""
+        try:
+            import mpmath as mp
+        except ModuleNotFoundError:
+            raise ModuleNotFoundError(
+                "Module mpmath is required for computations with model 'sh'."
+                )
+        self.hyp2f1 = np.frompyfunc(
+            lambda a, b, c, z: complex(mp.hyp2f1(a, b, c, z, maxterms=10**5)),
+            4, 1)
+
+    def legendreP(self, ell, z):
+        return self.hyp2f1(-ell, ell+1, 1, (1-z)/2)
+    def legendreP_prime(self, ell, z):
+        return ell*(ell+1)/2 * self.hyp2f1(-ell+1, ell+2, 2, (1-z)/2)
+
+    def legendreRatio(self, ell, z):
+        """
+        To avoid overflow, compute directly the ratio of Legendre functions.
+
+        .. math::
+            \\begin{aligned}
+            Q(l,z) & = \\dfrac{(1+z)  P'_l(z) }{ l(l+1) P_l(z)} \\\\
+            Q(l,z) & = \\dfrac{(1+z) F(-l+1, l+2, 2, (1-z)/2 )} \
+                                {2   F( -l,  l+1, 1, (1-z)/2 )}
+            \\end{aligned}
+
+        """
+        return (1+z)/2 * self.hyp2f1(-ell+1,ell+2,2,(1-z)/2) / self.hyp2f1(-ell, ell+1, 1, (1-z)/2)
+
+
+    def get_loss_flow_at(self, pipe, x, omegas_scaled):
+        conicity = pipe.get_conicity_at(x)
+        Theta = np.arctan(conicity)
+        cosTheta = np.cos(Theta)
+
+        rhos, mus = pipe.get_physics().get_coefs(x, 'rho', 'mu')
+        radius = calculate_hydraulic_radius(pipe, x)
+        omegas = omegas_scaled[:, np.newaxis] / pipe.get_scaling().get_time()
+
+        stokes = radius * np.sqrt(omegas*rhos/mus)
+        cTheta = (2*(1-cosTheta))/np.sin(Theta)
+
+        # Complex order of the spherical harmonics
+        eta = np.sqrt(-1j*stokes**2/cTheta**2 + 1/4) - 1/2
+
+        # Dimensionless loss coefficient
+        # Kv = self.legendreP_prime(eta, cosTheta) \
+        #         / (eta * (eta+1) * self.legendreP(eta, cosTheta)) \
+        #         * np.sin(Theta)**2 / (1-cosTheta)
+        Kv = self.legendreRatio(eta, cosTheta)
+
+        # Convert mpmath data to numpy
+        Kv = np.array(Kv, dtype=complex)
+        loss_flow = pipe.get_coef_flow_at(x) * Kv/(1-Kv)
+        return loss_flow
+
+    def get_loss_pressure_at(self, pipe, x, omegas_scaled):
+        conicity = pipe.get_conicity_at(x)
+        Theta = np.arctan(conicity)
+        cosTheta = np.cos(Theta)
+
+        rhos, Cp, kappa, gamma = pipe.get_physics().get_coefs(
+            x, 'rho', 'Cp', 'kappa', 'gamma')
+        radius = calculate_hydraulic_radius(pipe, x)
+        omegas = omegas_scaled[:, np.newaxis] / pipe.get_scaling().get_time()
+
+        sigmastokes = radius * np.sqrt(omegas*rhos*Cp/kappa)
+        cTheta = (2*(1-cosTheta))/np.sin(Theta)
+
+        # Complex order of the spherical harmonics
+        eta = np.sqrt(-1j*sigmastokes**2/cTheta**2 + 1/4) - 1/2
+        # Dimensionless loss coefficient
+        # Kt = self.legendreP_prime(eta, cosTheta) \
+        #         / (eta * (eta+1) * self.legendreP(eta, cosTheta)) \
+        #         * (1 + cosTheta)
+        Kt = self.legendreRatio(eta, cosTheta)
+
+        # Convert mpmath data to numpy
+        Kt = np.array(Kt, dtype=complex)
+        return pipe.get_coef_pressure_at(x) * (gamma - 1) * Kt
