@@ -1,15 +1,25 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
+
 """
 Defines openwind object used to design the bore profile. The radius evolution
 of each tube is described by a design shape parametrize by several design
@@ -20,8 +30,6 @@ parameters.
 from .design_parameter import (DesignParameter,
                                FixedParameter,
                                VariableParameter,
-                               VariableParameterSquare,
-                               VariableParameterLimitedRange,
                                VariableHolePosition,
                                VariableHoleRadius,
                                OptimizationParameters,
