@@ -298,6 +298,17 @@ class DesignParameter(ABC):
     def get_value(self):
         """Current geometric value of the parameter."""
 
+    @abstractmethod
+    def set_value(self, value):
+        """
+        set new geometric value of the parameter.
+
+        Parameters
+        ----------
+        value : float
+            The new value
+        """
+
     @abstractmethod
     def get_differential(self, diff_index):
         """
@@ -350,6 +361,9 @@ class FixedParameter(DesignParameter):
     def get_value(self):
         return self._value
 
+    def set_value(self, value):
+        self._value = value
+
     def get_differential(self, diff_index):
         return 0
 
@@ -411,6 +425,9 @@ class VariableParameter(DesignParameter):
     def get_value(self):
         return self._optim_params.get_param(self.index)
 
+    def set_value(self, value):
+        self._optim_params.values[self.index] = value
+
     def get_differential(self, diff_index):
         return self._optim_params.diff_param(self.index, diff_index)
 
@@ -486,6 +503,10 @@ class VariableHolePosition(DesignParameter):
         value = self._main_bore_shape.get_position_from_xnorm(x_norm)
         return value
 
+    def set_value(self, value):
+        norm_position = self._main_bore_shape.get_xnorm_from_position(value)
+        self._optim_params.values[self.index] = norm_position
+
     def get_differential(self, diff_index):
         d_zeta = self._optim_params.diff_param(self.index, diff_index)
         zeta = self._optim_params.get_param(self.index)
@@ -570,6 +591,12 @@ class VariableHoleRadius(DesignParameter):
         value = radius_main_pipe * zeta
         return value
 
+    def set_value(self, value):
+        x_norm = self.__get_x_norm()
+        radius_main_pipe = self._main_bore_shape.get_radius_at(x_norm)
+        zeta = value/radius_main_pipe
+        self._optim_params.values[self.index] = zeta
+
     def __get_x_norm(self):
         pos = self._hole_position.get_value()
         return self._main_bore_shape.get_xnorm_from_position(pos)
