@@ -217,10 +217,14 @@ class DesignShape(ABC):
 
         """
         tolerance = 1e-10
-        if np.any(x < bounds[0] - tolerance) or np.any(x > bounds[-1] + tolerance):
+        # if bounds[0] > bounds[1]:
+        #     raise AssertionError('The two bounds of the shape are inverted:'
+        #                           'x_up={:.2e} and x_down={:.2e} (should be: '
+        #                           'x_up < x_down)'.format(bounds[0], bounds[-1]))
+        if np.any(x < min(bounds) - tolerance) or np.any(x > max(bounds) + tolerance):
             raise AssertionError('A value is being estimated outside the shape!'
-                             'x in [{:.2e},{:.2e}], whereas bounds are '
-                             '[{:.2e},{:.2e}]'.format(np.min(x), np.max(x),
+                              'x in [{:.2e},{:.2e}], whereas bounds are '
+                              '[{:.2e},{:.2e}]'.format(np.min(x), np.max(x),
                                                       bounds[0], bounds[-1]))
 
     def is_TMM_compatible(self):
@@ -248,7 +252,10 @@ class DesignShape(ABC):
         """
         X_range = self.get_endpoints_position()
         Xmin, Xmax = eval_(list(X_range))
-        return Xmax - Xmin
+        length = Xmax - Xmin
+        if length < 0:
+            raise AssertionError('The pipe length is negative: {:.2e}'.format(length))
+        return length
 
     def get_diff_length(self, diff_index):
         """
@@ -280,6 +287,33 @@ class DesignShape(ABC):
         r = self.get_radius_at(x_norm)
         plt.plot(x, r, **kwargs)
 
+
+    def cut_shape(self, start, stop):
+        """
+        Cut a shape.
+
+        This cut lost the inversion outside the range and modify the desing
+        parameters associated to this shape. It is different than a ShapeSlice
+        which create a new DesignShape as a part of a mother shape (which is kept)
+
+        Parameters
+        ----------
+        start : float
+            DESCRIPTION.
+        stop : float
+            DESCRIPTION.
+
+        """
+        x = self.get_xnorm_from_position(np.array([start, stop]))
+        r_new = self.get_radius_at(x)
+        X0, X1 = self.get_endpoints_position()
+        X0.set_value(start)
+        X1.set_value(stop)
+        R0, R1 = self.get_endpoints_radius()
+        R0.set_value(r_new[0])
+        R1.set_value(r_new[1])
+
+
     # %% --- CONVERSION X /X_NORM METHODS ---
 
     def get_position_from_xnorm(self, x_norm):
