@@ -124,3 +124,6 @@ class ShapeSlice(DesignShape):
 
     def get_endpoints_radius(self):
         return 'The endpoints radii of a shape slice are not DesignParameters'
+
+    def cut_shape(self, start, stop):
+        raise ValueError('It is impossible to "cut" a ShapeSlice')
