@@ -490,3 +490,6 @@ class Spline(DesignShape):
         if np.isscalar(x):
             d_con = float(d_con)
         return d_con
+
+    def cut_shape(self, start, stop):
+        raise ValueError('It is not yet possible to slice a spline!')
