@@ -34,11 +34,15 @@ import scipy.linalg as la
 
 class GLQuad:
     """
-    Calculate the quantities for the GaussLobato quadrature.
-
-    This quadrature is based on the Gauss-Lobato technique applied to
-    Lagrangian polynomials taken on the precise quadrature points.
+    Calculate the quantities for the Gauss-Lobatto quadrature for 
+    Lagrange polynomial basis functions interpolated on the quadrature
+    points (spectral method).
 
+    see 
+    [1] Quarteroni, A., Sacco, R., & Saleri, F. (2008). 
+    Méthodes Numériques: Algorithmes, analyse et applications. 
+    Springer Science & Business Media.
+    
     Parameters
     ----------
     r : int
@@ -47,14 +51,13 @@ class GLQuad:
     Attributes
     ----------
     pts : array of r+1 float
-        the point position on the element
+        the point position on the reference element
     weigth : array of r+1 float
-        the weight of each node on this element
+        the weight of each point 
     omega : array of r + 1 float
         quantity to calculate integrals
     BK : square matrix (r+1, r+1)
-        quantity to calculate integrals
-
+        elementary matrix for gradient term
     """
 
     def __init__(self, r):
@@ -113,7 +116,7 @@ class GLQuad:
         return p
 
     def lagran_polys_derivate(self):
-        """Compute de derivate of the lagrange polynomials
+        """Compute de derivate of the Lagrange interpolation polynomials
         """
         positions = self.pts
 
