@@ -102,19 +102,40 @@ class FrequentialSolver:
 
     """
 
+    FMIN_disc = 2000.0 # provides a mesh adapted to at least FMIN_disc Hz
+    """
+    float
+    The minimal frequency in Hz, for which the mesh is adapted. The mesh is
+    adapted to the frequency max([frequencies, FMIN_disc])
+    """
+
     def __init__(self, instru_physics, frequencies, diffus_repr_var=False,
                  note=None, compute_method='FEM', **discr_params):
         self.netlist = instru_physics.netlist
+
+        # option
         self.discr_params = discr_params
         self.diffus_repr_var = diffus_repr_var   # When using 'diffrepr+'
         self.compute_method = compute_method   # FEM, TMM, or hybrid?
 
+        # set frequencial axis and options for automatic meshing
         frequencies = self._check_frequencies(frequencies, compute_method)
+        self._update_shortestLbd(frequencies)
         self.frequencies = frequencies
+
+        # discretize the netlist components and set their frequencial properties
         self._convert_frequential_components()
+
+        # organize the components in the big matrix
         self._organize_components()
+
+        # construct the matrix of the pipes (independant of the fingering)
         self._construct_matrices_pipes()
+
+        # set the fingering
         self._set_note(note)
+
+        # construct the matrix of the connectors
         self._construct_matrices()
 
     def __repr__(self):
@@ -160,6 +181,29 @@ class FrequentialSolver:
         """
         return self.impedance
 
+    def _update_shortestLbd(self, frequencies):
+        """
+        Update the value of the discretization parameter 'shortestLbd': the
+        shortest wavelength to considered in case of automatic meshing, with
+        respect to the value of the highest frequency
+        """
+        con, end = self.netlist.get_connector_and_ends('source')
+        phy_entrance = end[0].pipe.get_physics()
+        c_entrance = phy_entrance.get_coefs(0, 'c')[0]
+        FMAX = np.max([np.max(frequencies), FrequentialSolver.FMIN_disc])
+        if 'shortestLbd' not in self.discr_params.keys():
+            self.discr_params['shortestLbd'] = c_entrance / FMAX
+            is_updated = True
+        elif FMAX != c_entrance/self.discr_params['shortestLbd']:
+            self.discr_params['shortestLbd'] = c_entrance / FMAX
+            warnings.warn('The shortest wave length used for automatic meshing'
+                          ' has been changed to fit the highest frequency: '
+                          'lambda = {:.2e} m'.format(self.discr_params['shortestLbd']))
+            is_updated = True
+        else:
+            is_updated = False
+        return is_updated
+
     @staticmethod
     def _check_frequencies(frequencies, compute_method):
         """
@@ -219,7 +263,10 @@ class FrequentialSolver:
 
         """
         self.frequencies = self._check_frequencies(frequencies, self.compute_method)
-        self._organize_components()
+        if self._update_shortestLbd(frequencies):
+            # update mesh if the shortest wavelength has been modified
+            self._convert_frequential_components()
+            self._organize_components()
         self._construct_matrices_pipes()
         self._construct_matrices()
 
@@ -323,7 +370,7 @@ class FrequentialSolver:
         raise ValueError("Could not convert connector %s" % str(connector))
 
 
-    def _convert_frequential_components(self, **kwargs):
+    def _convert_frequential_components(self):
         self.f_pipes, self.f_connectors = \
             self.netlist.convert_with_structure(self._convert_pipe,
                                                 self._convert_connector)
@@ -603,7 +650,7 @@ class FrequentialSolver:
 
     # %% --- Plotting functions ---
 
-    def plot_flow(self, freq=0):
+    def plot_flow_at_freq(self, freq, **kwargs):
         """
         Plot the acoustic flow for a given frequency inside the instrument.
 
@@ -612,15 +659,19 @@ class FrequentialSolver:
 
         Parameters
         ----------
-        freq : float, optional
+        freq : float
             The expected frequency (it uses the first higher frequency computed).
-            The default is 0.
+        **kwargs : key word arguments
+            Passed to `plt.plot() <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.plot.html>`_.
 
         """
         ifreq = np.where(self.frequencies >= freq)[0][0]
-        plt.plot(self.x_interp, np.real(self.flow[ifreq]))
+        plt.plot(self.x_interp, np.real(self.flow[ifreq]), **kwargs)
+        plt.xlabel('Position (m)')
+        plt.ylabel('Flow (m/s)')
+        plt.legend()
 
-    def plot_pressure(self, freq=0):
+    def plot_pressure_at_freq(self, freq, **kwargs):
         """
         Plot the acoustic pressure for a given frequency inside the instrument.
 
@@ -629,15 +680,19 @@ class FrequentialSolver:
 
         Parameters
         ----------
-        freq : float, optional
+        freq : float
             The expected frequency (it uses the first higher frequency computed).
-            The default is 0.
+        **kwargs : key word arguments
+            Passed to `plt.plot() <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.plot.html>`_.
 
         """
         ifreq = np.where(self.frequencies >= freq)[0][0]
-        plt.plot(self.x_interp, np.real(self.pressure[ifreq]))
+        plt.plot(self.x_interp, np.real(self.pressure[ifreq]), **kwargs)
+        plt.xlabel('Position (m)')
+        plt.ylabel('Pressure (Pa)')
+        plt.legend()
 
-    def plot_impedance(self, **kwargs):
+    def plot_impedance(self, normalize=True, **kwargs):
         """
         Plot the normalized impedance.
 
@@ -645,22 +700,26 @@ class FrequentialSolver:
 
         Parameters
         ----------
+        normalize : bool, optional
+            Normalize or not the impedance by the input characteristic
+            impedance. The default is True.
         **kwargs : keyword arguments
             They are transmitted to :py:func:`plot_impedance()\
             <openwind.impedance_tools.plot_impedance>`.
 
         """
+        if normalize:
+            Zc = self.get_ZC_adim()
+        else:
+            Zc=1
+
         tools.plot_impedance(self.frequencies, self.impedance,
-                             self.get_ZC_adim(), **kwargs)
+                             Zc, **kwargs)
 
-    def plot_var3D(self, dbscale=True, var='pressure'):
+    def plot_var3D(self, dbscale=True, var='pressure', with_plotly=False):
         """
-        Plot with a 3D surface one acoustic field in the instrument for every
-        frequency.
-
-        .. warning::
-            This method uses the external module `plotly` which must be
-            installed by the user.
+        Plot one acoustic field in the instrument for every
+        frequency on a surface.
 
         Parameters
         ----------
@@ -668,40 +727,51 @@ class FrequentialSolver:
             Plot the fields with a dB scale or not. The default is True.
         var : 'pressure' or 'flow', optional
             Which field must be plotted. The default is 'pressure'.
+        **kwargs : key word arguments
+            Passed to `plt.pcolor() <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.pcolor.html>`_.
 
         """
+        X = self.x_interp
+        Y = self.frequencies
+        if var == 'pressure':
+            Z = self.pressure
+
+        elif var == 'flow':
+            Z = self.flow
+
+        else:
+            raise ValueError("possible values are pressure or flow")
+        if dbscale:
+            Zplot = 20*np.log10(np.abs(Z))
+            label = '{} [dB]'.format(var)
+        else:
+            Zplot = np.real(Z)
+            label = 'Real({})'.format(var)
+
+        if with_plotly:
+            self._plot_var3d_plotly(X, Y, Zplot, label)
+        else:
+            self._plot_var3d_pcolor(X, Y, Zplot, label)
+
+    @staticmethod
+    def _plot_var3d_plotly(X, Y, Z, label):
+        """Plot with a 3D surface using plotly"""
+
+        filename = label + '_3D.html'
         try:
             import plotly.graph_objs as go
             import plotly.offline as py
         except ImportError as err:
             msg = "Function plot_var3D() requires plotly."
             raise ImportError(msg) from err
-
-        """Plot either the pressure or the flow for every frequency inside the
-        entire instrument.
-        """
-        X = self.x_interp
-        Y = self.frequencies
-        if var == 'pressure':
-            Z = self.pressure
-            filename = 'pressure_3D.html'
-        elif var == 'flow':
-            Z = self.flow
-            filename = 'flow_3D.html'
-        else:
-            raise ValueError("possible values are pressure or flow")
-        if dbscale:
-            Zplot = 20*np.log10(np.abs(Z))
-        else:
-            Zplot = np.real(Z)
         try:
-            layout_3D = go.Layout(scene=dict(xaxis=dict(title='Position',
+            layout_3D = go.Layout(scene=dict(xaxis=dict(title='Position (m)',
                                                         autorange='reversed'),
-                                             yaxis=dict(title='Frequency',
+                                             yaxis=dict(title='Frequency (Hz)',
                                                         autorange='reversed'),
-                                             zaxis=dict(title='Field')))
+                                             zaxis=dict(title=label)))
 
-            data_u3D = [go.Surface(x=X, y=Y, z=Zplot,
+            data_u3D = [go.Surface(x=X, y=Y, z=Z,
                                    contours=go.surface.Contours(
                                            x=go.surface.contours.X(
                                                    highlightcolor="#42f462",
@@ -717,8 +787,17 @@ class FrequentialSolver:
         except:
             print('Impossible to load plotly: no 3D plot')
 
-    def plot_acoustics_fields(self, notes, variable='power', logscale=False,
-                              scaled=False, **kwargs):
+
+    @staticmethod
+    def _plot_var3d_pcolor(X, Y, Z, label, **kwargs):
+        plt.figure()
+        im = plt.pcolor(X, Y, Z, shading='auto', **kwargs)
+        plt.colorbar(im, label=label)
+        plt.xlabel('Position (m)')
+        plt.ylabel('Frequency (Hz)')
+
+    def plot_norm_ac_fields_at_notes(self, notes, variable='power', logscale=False,
+                                     scaled=False, **kwargs):
         """
         A map plotting the norm of an acoustic quantity at each interpolation
         point for each note.
