@@ -184,13 +184,6 @@ class ImpedanceComputation:
     """
 
 
-    FMIN_disc = 2000.0 # provides a mesh adapted to at least FMIN_disc Hz
-    """
-    float
-    The minimal frequency in Hz, for which the mesh is adapted. The mesh is
-    adapted to the frequency max([frequencies, FMIN_disc])
-    """
-
     def __init__(self, frequencies, main_bore, holes_valves=None,
                  fingering_chart=None, player = None, temperature=None,
                  losses=True,
@@ -226,13 +219,11 @@ class ImpedanceComputation:
                                       discontinuity_mass=discontinuity_mass,
                                       matching_volume=matching_volume)
 
-        FMAX = np.max([np.max(frequencies), ImpedanceComputation.FMIN_disc])
-        shortest_lambda = 346.3 / FMAX
+
 
         kwargs = {'diffus_repr_var':diff_repr_vars,
                   'l_ele':l_ele,
                   'order':order,
-                  'shortestLbd':shortest_lambda,
                   'note':note,
                   'nb_sub':nb_sub}
 
