@@ -166,7 +166,7 @@ def convert_frequencies_temperature(frequencies, temperature_in, temperature_out
     return freq_ref
 
 def plot_impedance(frequencies, impedance, Zc0=1, dbscale=True, figure=None,
-                   label=None, **kwargs):
+                   label=None, modulus_only=False, **kwargs):
     """
     Plot the impedance of the instrument.
 
@@ -184,6 +184,8 @@ def plot_impedance(frequencies, impedance, Zc0=1, dbscale=True, figure=None,
         Where to plot. By default, opens a new figure.
     label : str, optional
         The label associated to the curve. Default is None
+    modulus_only: bool, optional
+        If true, only the modulus of the impedance is plotted
     **kwargs : key word arguments
         Passed to `plt.plot() <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.plot.html>`_.
     """
@@ -197,11 +199,13 @@ def plot_impedance(frequencies, impedance, Zc0=1, dbscale=True, figure=None,
     if Zc0 != 1:
         ylabel += '/Zc'
 
-    if len(ax) < 2:
+    if len(ax) < 1 and not modulus_only:
         ax = [fig.add_subplot(2, 1, 1)]
         ax.append(fig.add_subplot(2, 1, 2, sharex=ax[0]))
-        ax[0].grid()
-        ax[1].grid()
+    elif len(ax) < 1:
+        ax.append(fig.add_subplot(1, 1, 1))
+    [a.grid() for a in ax]
+
     if dbscale:
         ax[0].plot(frequencies, 20*np.log10(np.abs(impedance/Zc0)), label=label, **kwargs)
         ylabel += ' (dB)'
@@ -210,16 +214,19 @@ def plot_impedance(frequencies, impedance, Zc0=1, dbscale=True, figure=None,
         ax[0].plot(frequencies, np.abs(impedance/Zc0), label=label, **kwargs)
         ax[0].set_ylabel("|Z|")
     ax[0].set_ylabel(ylabel)
+    ax[0].grid('on')
     if label:
         ax[0].legend(loc='upper right')
 
-    ax[1].plot(frequencies, np.angle(impedance), **kwargs)
-    ax[1].set_xlabel("Frequency (Hz)")
-    ax[1].set_ylabel("angle(Z)")
-    ax[1].get_yaxis().set_ticks([-np.pi/2, 0, np.pi/2])
-    ax[1].get_yaxis().set_ticklabels(
-        ['$-\pi/2$', '0', '$\pi/2$']
-    )
+    if len(ax)>1 and not modulus_only:
+        ax[1].plot(frequencies, np.angle(impedance), **kwargs)
+        ax[1].grid('on')
+        ax[1].set_xlabel("Frequency (Hz)")
+        ax[1].set_ylabel("angle(Z)")
+        ax[1].get_yaxis().set_ticks([-np.pi/2, -np.pi/4, 0, np.pi/4, np.pi/2])
+        ax[1].get_yaxis().set_ticklabels(
+            ['$-\pi/2$', '$-\pi/4$', '0', '$-\pi/4$', '$\pi/2$']
+        )
 
 def plot_reflection(frequencies, impedance, Zc0,
                     complex_plane=True, figure=None, **kwargs):
@@ -377,3 +384,15 @@ def find_peaks_measured_impedance(frequencies, impedance, Npeaks=10, fmin=0):
         polf = np.polyfit(x, y, 1)
         res_freq[k] = -polf[1]/polf[0]
     return res_mag, res_freq
+
+def get_equal_temperament_frequency(midi, tuning=440, nb_harmo=1):    
+    # midi 69 corresponds to {tuning} Hz
+    r = np.power(2,1/12)
+    midi = np.array(midi)
+    freq = tuning*np.power(r,midi-69)
+    if(midi.size>1 and nb_harmo>1):
+        print("Harmonics only implemented for one midi number")
+    freqs = (np.arange(0,nb_harmo)+1)*freq
+    return freqs
+
+
