@@ -28,7 +28,7 @@ from openwind.impedance_tools import plot_impedance
 from openwind import InstrumentGeometry
 
 
-def plot_evolution_geometrie(inverse, params_evol, target_geom=None,
+def plot_evolution_geometry(inverse, params_evol, target_geom=None,
                              print_fig=False, save_name='', double_plot=True,
                              title='', xlim=None, ylim=None, **kwargs):
     """
