@@ -260,7 +260,7 @@ class InverseFrequentialResponse(FrequentialSolver):
 
         self.optim_params.set_active_values(new_optim_values)
         self.instru_physics.update_netlist()
-        self._convert_frequential_components(**self.discr_params)
+        self._convert_frequential_components()
         self._organize_components()
         self._construct_matrices_pipes()
         self._construct_matrices()
