@@ -26,7 +26,7 @@ Read and transcript the data from user format to openwind data.
 """
 
 from .fingering_chart import FingeringChart, Fingering
-from .instrument_geometry import InstrumentGeometry
+from .instrument_geometry import InstrumentGeometry, BrassValve
 from .score import Score
 from .adjust_instrument_geometry import AdjustInstrumentGeometry
 from .player import Player
