@@ -311,17 +311,17 @@ class InstrumentGeometry:
         Xlast = None
         Rlast = None
         for k, raw_part in enumerate(main_bore_list):
-            label = 'bore_' + str(k)
+            label = 'bore' + str(k)
             X, R, shape_type, Geom_param = self._parse_geometry(raw_part, Xlast,
                                                                 Rlast, label)
 
             if Xlast is not None and \
                 min([x.get_value() for x in X]) < Xlast.get_value():
-                raise ValueError("Some abscissae x are going backwards, "
+                raise ValueError(f"{label}: Some abscissae x are going backwards, "
                                  "there must be a mistake in the instrument"
                                  " file.")
             if min([r.get_value() for r in R]) < 0:
-                raise ValueError("Radius must be positive.")
+                raise ValueError(f"{label}: Radius must be positive.")
 
             if shape_type is not None:
                 new_shape = self._build_shape(X, R, shape_type, Geom_param)
@@ -962,7 +962,7 @@ class InstrumentGeometry:
         elif np.isclose(self.__estim_value(raw_part[0]), Xlast.get_value()):  # the parts of the main bore must be connected
             X = [Xlast]
         else:
-            msg = ('The main bore parts are not connected! '
+            msg = (f'{label}: The main bore parts are not connected! '
                    'It is impossible to construct the instrument.')
             raise ValueError(msg)
 
@@ -973,6 +973,8 @@ class InstrumentGeometry:
             R = [Rlast]
         shape_type = raw_part[4].lower()
         if shape_type == 'spline':
+            if len(raw_part) < 7:
+                raise ValueError(f'{label}: "Spline" shape needs at least 2 supplementary parameters.')
             params = raw_part[5:]
             N = len(params)//2
             X.extend([self.__designparameter(value, label + '_spline_x' + str(k))
@@ -987,6 +989,8 @@ class InstrumentGeometry:
         X.append(self.__designparameter(raw_part[1], label + '_pos_plus'))
         R.append(self.__designparameter(raw_part[3], label + '_radius_plus'))
         if shape_type == 'circle' or shape_type == 'bessel':
+            if len(raw_part) != 6:
+                raise ValueError(f'{label}: "Circle" and "Bessel" shapes need one supplementary parameter.')
             Geom_param = [self.__designparameter(raw_part[5], label + '_param')]
         else:
             Geom_param = []
