@@ -28,7 +28,7 @@ from openwind.continuous import (ThermoviscousLossless,
                                  ThermoviscousDiffusiveRepresentation,
                                  RadiationPade,
                                  PhysicalRadiation,
-                                 JunctionTjoint, JunctionSimple,
+                                 JunctionTjoint, JunctionSimple, JunctionSwitch,
                                  JunctionDiscontinuity,
                                  Reed1dof, Flow,
                                  RadiationPerfectlyOpen
@@ -36,6 +36,7 @@ from openwind.continuous import (ThermoviscousLossless,
 from openwind.temporal import (TemporalPipe, TemporalRadiation,
                                TemporalJunction, TemporalLossyPipe,
                                TemporalSimpleJunction,
+                               TemporalJunctionSwitch,
                                TemporalJunctionDiscontinuity,
                                TemporalFlowCondition,
                                TemporalReed1dof,
@@ -183,6 +184,8 @@ class TemporalSolver:
         if isinstance(connector, JunctionDiscontinuity):
             return TemporalJunctionDiscontinuity(connector, ends,
                                                  t_solver=self)
+        if isinstance(connector, JunctionSwitch):
+            return TemporalJunctionSwitch(connector, ends, t_solver=self)
 
         if isinstance(connector, Reed1dof):
             return TemporalReed1dof(connector, ends, t_solver=self,
@@ -312,8 +315,7 @@ class TemporalSolver:
         :py:meth:`run_simulation_steps`:
             To run for a given number of steps instead.
         """
-        self._execute_score.set_score(self.instru_physics.player.get_score())
-        self.instru_physics._update_player()
+
         n_steps_needed = int(np.ceil(duration / self._dt))
         if n_steps is not None:
             print("Custom number of steps:", n_steps)
@@ -355,6 +357,8 @@ class TemporalSolver:
         run_simulation:
             To run for a given duration instead.
         """
+        self._execute_score.set_score(self.instru_physics.player.get_score())
+        self.instru_physics._update_player()
         self.tracker = SimulationTracker(n_steps, display_enabled=enable_tracker_display)
         if energy_check:
             self.energy_check = EnergyCheck(self)
