@@ -213,7 +213,7 @@ class TemporalJunctionDiscontinuity(TemporalComponent):
 
         r_1, rho = self._end1.get_physical_params()[0:2]
         r_2 = self._end2.get_physical_params()[0]
-        self.m_disc = junct.compute_masse(r_1, r_2, rho)
+        self.m_disc = junct.compute_mass(r_1, r_2, rho)
 
         self.reset_variables()
         self._should_recompute_constants = True
@@ -302,3 +302,160 @@ class TemporalJunctionDiscontinuity(TemporalComponent):
 
     def get_maximal_dt(self):
         return np.infty
+
+
+class TemporalJunctionSwitch(TemporalComponent):
+    """
+    Switch Junction between 3 Pipes with acoustic mass related to cross-section
+    discontinuity for time-domain simulation.
+
+    Parameters
+    ----------
+    junct : :py:class:`JunctionSwitch<openwind.continuous.junction.JunctionSwitch>`
+        The continuous version of the junction
+
+    ends : list of 3 :py:class:`TemporalPipeEnd<openwind.temporal.tpipe.TemporalPipeEnd>`
+        The three pipe end objects the junction connects in the order
+
+        - 1 the always connected end
+        - 2 the end connected when the switch is "open"
+        - 3 the end connected when the switch is "closed"
+
+    t_solver : :py:class:`TemporalSolver<openwind.temporal.temporal_solver.TemporalSolver>`
+        The solver instance
+
+    Attributes
+    ----------
+    M_J : 2D array
+        The  mass matrix related to the acoustic masses due to cross section
+        discontinuity at the junction
+    T_J : 2D array
+        Transmission matrix.
+    dt: float
+        the time step
+    """
+
+    def __init__(self, junct, ends, t_solver):
+        super().__init__(junct, t_solver)
+        self._end1, self._end2, self._end3 = ends
+        self._junct = junct
+        self._opening_factor = 1.0
+        self.update_coefficients()
+        self.reset_variables()
+        self._should_recompute_constants = True
+
+    def set_dt(self, dt):
+        self.dt = dt
+        self._should_recompute_constants = True
+
+    def reset_variables(self):
+        """
+        Reinitialize all variables to start the simulation over.
+        """
+        self._gamma = np.zeros(1)
+        self._P_corr = np.zeros(3)
+
+    def update_coefficients(self):
+        """
+        Update junction coefficients when the opening factor is modified.
+
+        Update :py:attr:`M_J` and :py:attr:`T_J`.
+        """
+        r1, rho = self._end1.get_physical_params()[0:2]
+        r2 = self._end2.get_physical_params()[0]
+        r3 = self._end3.get_physical_params()[0]
+        self.M_J, self.T_J = self._junct.compute_masses(r1, r2, r3, rho,
+                                                        self._opening_factor)
+        if self.M_J != 0:
+            raise ValueError('The brass valve junction does not accept yet '
+                              'discontinuity mass in the temporal domain.'
+                              ' Please set the option "discontinuity_mass" to False.')
+        self._should_recompute_constants = True
+
+    def set_opening_factor(self, opening_factor):
+        """Control opening and closing of the switch.
+
+        Can be used during simulation. Energy-safe.
+
+        Parameters
+        ----------
+        opening_factor : float
+            1 for a raised valve (open switch) connect pipe 1 to 2,
+            0 for a depressed valve  (closed switch) connect pipe 1 to 3,
+            or any value in-between for a partly-closed switch
+        """
+        # Only update the factor if it changed significantly : too expensive (5,92 sec pour set_op contre 4,44)
+        if not (abs(opening_factor - self._opening_factor) < 1e-5):
+            self._opening_factor = opening_factor
+            self.update_coefficients()
+
+    def _precompute_constants(self):
+        """Compute constant matrices used in the update"""
+        m_end1 = self.dt / (2 * self._end1.get_alpha())
+        m_end2 = self.dt / (2 * self._end2.get_alpha())
+        m_end3 = self.dt / (2 * self._end3.get_alpha())
+        invM_end = diags([1/m_end1, 1/m_end2, 1/m_end3])
+
+        # Matrix of self-contribution of gamma trough the evolution of pressure
+        A_g = self.T_J @ invM_end @ self.T_J.T
+        # Matrix M_{J,dt}, to invert for performing a half-timestep
+        M_Jdt = self.M_J + self.dt**2/4 * A_g
+        invM_Jdt = np.array(inv(M_Jdt))
+        # Matrix used to update gamma by half a step
+        self.Step = invM_Jdt @ self.M_J
+        # Influence of each pipe's pressure and velocity on the update of gamma
+        self.Infl = -self.dt/2 * invM_Jdt @ self.T_J
+        self._should_recompute_constants = False
+
+    def _read_end_values(self):
+        # Predicted value of P at time n+1/2
+        self._P_corr[0] = self._end1.get_p_no_flow()
+        self._P_corr[1] = self._end2.get_p_no_flow()
+        self._P_corr[2] = self._end3.get_p_no_flow()
+
+    def _update_flows(self, gamma_b):
+        u = -self.T_J*gamma_b[0] #flow convention
+        self._end1.update_flow(u[0, 0])
+        self._end2.update_flow(u[0, 1])
+        self._end3.update_flow(u[0, 2])
+
+    def one_step(self):
+        """Advance one time step
+
+        Computes the evolution of internal variable ``gamma``,
+        and updates the flow on the three connected
+        :py:class:`TemporalPipeEnd<openwind.temporal.tpipe.TemporalPipeEnd>`.
+        """
+        if self._should_recompute_constants:
+            self._precompute_constants()
+
+        self._read_end_values()
+        gamma = self._gamma
+        gamma_b = self.Step @ gamma + self.Infl @ self._P_corr
+        self._gamma = 2*gamma_b - gamma
+        self._update_flows(gamma_b)
+
+    def __str__(self):
+        name = "TJunctionSwitch({}, {})"
+        return name.format(self._end1, self._end2)
+
+    def __repr__(self):
+        return self.__str__()
+
+    def energy(self):
+        """
+        Compute the amount of energy stored in the junction at time step
+
+        Returns
+        -------
+        float : the stored energy
+
+        """
+        energy  = self.M_J[0]/2 * self._gamma[0]**2
+        return energy
+
+    def dissipated_last_step(self):
+        return 0
+
+    def get_maximal_dt(self):
+        return np.infty
