@@ -28,6 +28,8 @@ import numpy as np
 from scipy.sparse import diags
 from scipy.sparse.linalg import eigs
 from scipy.linalg import eig
+# Low-level solution to avoid overhead every time we do a matrix-vector multiplication
+from scipy.sparse._sparsetools import csr_matvec
 
 from openwind.continuous import EndPos
 from openwind.discretization import DiscretizedPipe
@@ -55,10 +57,13 @@ class TemporalPipeEnd:
 
     """
 
+    __slots__ = ["t_pipe", "pos", "_w_nph", "_updated", "_alpha", "_array_pos"]
+
     def __init__(self, t_pipe, pos):
         self.t_pipe = t_pipe
         assert isinstance(pos, EndPos)
         self.pos = pos
+        self._array_pos = pos.array_pos
         self.reset_variables()
 
     def reset_variables(self):
@@ -123,7 +128,7 @@ class TemporalPipeEnd:
         float
             The value of :math:`p_{\text{no flow}}`
         """
-        return self.t_pipe.get_p_no_flow(self.pos.array_pos)
+        return self.t_pipe.get_p_no_flow(self._array_pos)
 
     def get_q_nph(self):
         r"""
@@ -461,9 +466,19 @@ class TemporalPipe(DiscretizedPipe, TemporalComponent):
         # HEAVY COMPUTATIONS HAPPEN HERE:
         # *******************************
         # Flow update
-        V = V_old + self.dtinvML2B * P
+        # V = V_old + self.dtinvML2B * P
+        V = V_old.copy()
+        csr_matvec(self.nL2, self.nH1, self.dtinvML2B.indptr,
+                   self.dtinvML2B.indices,
+                   self.dtinvML2B.data, P,
+                   V)
         # Prepare next pressure update
-        self.dtinvMBtV = self.dtinvMBt * V
+        # self.dtinvMBtV = self.dtinvMBt * V
+        self.dtinvMBtV[:] = 0
+        csr_matvec(self.nH1, self.nL2, self.dtinvMBt.indptr,
+                   self.dtinvMBt.indices,
+                   self.dtinvMBt.data, V,
+                   self.dtinvMBtV)
 
         # if check_scheme:
         #     dt = self._dt
