@@ -26,6 +26,9 @@ Numerical scheme for a Pipe with losses.
 """
 import numpy as np
 from scipy.sparse import diags
+from scipy.sparse.csr import csr_matrix
+# Low-level solution to avoid overhead every time we do a matrix-vector multiplication
+from scipy.sparse._sparsetools import csr_matvec
 
 from openwind.continuous import ThermoviscousDiffusiveRepresentation
 from openwind.temporal import TemporalPipe
@@ -76,6 +79,37 @@ class TemporalLossyPipe(TemporalPipe):
 
     """
 
+    __slots__ = ["_diffrepr_coefs",
+                  "_loss_N",
+                  "nH1", "nL2",
+                  "PV",
+                  "_P0", "_Pi",
+                  "_P_prev", "P0_prev", "_Pi_prev",
+                  "_Vi", "_V_prev", "_Vi_prev",
+                  "_V_prevprev",
+                  "_next_p_no_flow",
+                  "mH1", "mL2", "Bh",
+                  "MH1inv", "ML2inv",
+                  "dtinvMBt", "dtinvML2B",
+                  "energy_matrix",
+                  "end_minus",
+                  "end_plus",
+                  "p_to_p_noflow",
+                  "p0_to_p_noflow",
+                  "pi_to_p_noflow",
+                  "v_to_p_noflow",
+                  "p_to_p0",
+                  "p0_to_p0",
+                  "pi_to_p0",
+                  "p_to_pi",
+                  "pi_to_pi",
+                  "v_to_v",
+                  "vi_to_v",
+                  "p_to_v",
+                  "v_to_vi",
+                  "vi_to_vi",
+                  ]
+
     def __init__(self, pipe, t_solver, **params):
         super().__init__(pipe, t_solver, **params)
         assert isinstance(pipe.get_losses(), ThermoviscousDiffusiveRepresentation)
@@ -123,7 +157,8 @@ class TemporalLossyPipe(TemporalPipe):
         self.p0_to_p_noflow = Gamma_3 / denom_P
         self.pi_to_p_noflow = gi_2 / Gamma_2 * self.p0_to_p_noflow
         assert np.allclose(self.pi_to_p_noflow, gi_2*c0_/(c0_+Gamma_2)/denom_P)
-        self.v_to_p_noflow = - diags(1/(2*denom_P)) @ self.Bh.T
+        self.v_to_p_noflow = - diags(1/(2*denom_P)) @ self.Bh.T  # csr sparse matrix
+        assert isinstance(self.v_to_p_noflow, csr_matrix)
         self.end_minus.set_alpha(1/(2 * denom_P[0]))
         self.end_plus.set_alpha(1/(2 * denom_P[-1]))
 
@@ -154,6 +189,7 @@ class TemporalLossyPipe(TemporalPipe):
         self.v_to_v = (mV_ - Psi_) / denom_V
         self.vi_to_v = 2 * ri_2 / denom_V
         self.p_to_v = diags(1/denom_V) @ self.Bh
+        assert isinstance(self.p_to_v, csr_matrix)
         self.v_to_vi = ri_ / (li_ + ri_)
         self.vi_to_vi = (li_ - ri_) / (li_ + ri_)
 
@@ -210,12 +246,45 @@ class TemporalLossyPipe(TemporalPipe):
         P_nph[0] = self.end_minus.accept_q_nph()
         P_nph[-1] = self.end_plus.accept_q_nph()
 
-        P_next = 2*P_nph - P
-        P0_next = self.p_to_p0 * (P + P_next) + self.p0_to_p0 * P0 + np.sum(self.pi_to_p0 * Pi, axis=0)
-        Pi_next = self.p_to_pi * (P + P_next - P0 - P0_next) + self.pi_to_pi * Pi
+        # P_next = 2*P_nph - P
+        # P0_next = self.p_to_p0 * (P + P_next) + self.p0_to_p0 * P0 + np.add.reduce(self.pi_to_p0 * Pi, axis=0)
+        # Pi_next = self.p_to_pi * (P + P_next - P0 - P0_next) + self.pi_to_pi * Pi
+        # # Update of V
+        # V_next = self.v_to_v * V + np.add.reduce(self.vi_to_v * Vi, axis=0) + self.p_to_v @ P_next
+        # Vi_next = self.v_to_vi * (V + V_next) + self.vi_to_vi * Vi
+
+        # Limit dynamic memory allocation by using in-place operations
+
+        P_next = 2*P_nph
+        P_next -= P
+
+        # P0_next = self.p_to_p0 * (P + P_next) + self.p0_to_p0 * P0 + np.sum(self.pi_to_p0 * Pi, axis=0)
+        P0_next = P+P_next
+        P0_next *= self.p_to_p0
+        P0_next += self.p0_to_p0 * P0
+        # P0_next += np.sum(self.pi_to_p0 * Pi, axis=0)
+        P0_next += np.add.reduce(self.pi_to_p0 * Pi, axis=0) # Slightly more efficient?
+
+        # Pi_next = self.p_to_pi * (P + P_next - P0 - P0_next) + self.pi_to_pi * Pi
+        Pi_next = self.p_to_pi * (P + P_next - P0 - P0_next)
+        Pi_next += self.pi_to_pi * Pi
+
         # Update of V
-        V_next = self.v_to_v * V + np.sum(self.vi_to_v * Vi, axis=0) + self.p_to_v @ P_next
-        Vi_next = self.v_to_vi * (V + V_next) + self.vi_to_vi * Vi
+        # V_next = self.v_to_v * V + np.sum(self.vi_to_v * Vi, axis=0) + self.p_to_v @ P_next
+        # V_next = np.sum(self.vi_to_v * Vi, axis=0)
+        V_next = np.add.reduce(self.vi_to_v * Vi, axis=0)
+        # V_next += self.p_to_v * P_next # /!\ Sparse matrix - vector multiplication
+        csr_matvec(self.nL2, self.nH1, self.p_to_v.indptr,
+                   self.p_to_v.indices,
+                   self.p_to_v.data, P_next,
+                   V_next)
+        V_next += self.v_to_v * V
+
+        # Vi_next = self.v_to_vi * (V + V_next) + self.vi_to_vi * Vi
+        Vi_next = self.v_to_vi * (V + V_next)
+        Vi_next += self.vi_to_vi * Vi
+
+
 
 #        if check_scheme:
 #            dt = self._dt
@@ -254,9 +323,22 @@ class TemporalLossyPipe(TemporalPipe):
         boundary flow.
         """
         P, V = self.PV
+        # self._next_p_no_flow = (self.p_to_p_noflow * P + self.p0_to_p_noflow * self._P0 +
+        #             np.add.reduce(self.pi_to_p_noflow * self._Pi, axis=0)
+        #             + self.v_to_p_noflow * V) # /!\ Sparse matrix - vector multiplication
+
         self._next_p_no_flow = (self.p_to_p_noflow * P + self.p0_to_p_noflow * self._P0 +
-                    np.sum(self.pi_to_p_noflow * self._Pi, axis=0)
-                    + self.v_to_p_noflow @ V)
+                    np.add.reduce(self.pi_to_p_noflow * self._Pi, axis=0))
+
+        # Low-level call for efficient sparse matrix-vector multiplication
+        # csr_matmul(M, N, indptr, indices, data, other, result)
+        # computes Y += A*X
+        assert self.v_to_p_noflow.shape == (self.nH1, self.nL2)
+        csr_matvec(self.nH1, self.nL2, self.v_to_p_noflow.indptr,
+                   self.v_to_p_noflow.indices,
+                   self.v_to_p_noflow.data, V,
+                   self._next_p_no_flow)
+
 #        assert np.allclose(self._next_p_no_flow, (P + self._compute_P_next())/2)
 
 #    def _compute_P_next(self):
